@@ -77,7 +77,8 @@ typedef struct switches_l {
 	T_STRING,
 	T_UINT,
 	T_UINT8,
-	T_UINT16
+	T_UINT16,
+	T_UQSTRING
     } type;
 } switches_t;
 
@@ -175,7 +176,8 @@ static switches_t dswitches[]= {
     { "HIDDEN", OFFS(hidden), T_UINT },
     { "PRECMD", OFFS(precmd), T_STRING },
     { "BLOCKSIZE", OFFS(blocksize), T_UINT },
-    { "CODEPAGE", OFFS(codepage), T_UINT }
+    { "CODEPAGE", OFFS(codepage), T_UINT },
+    { "DATA_MAP", OFFS(data_map), T_UQSTRING }
 };
 
 #if (defined  HAVE_TOUPPER_L || defined HAVE_STRNCASECMP_L)
@@ -201,12 +203,12 @@ static int canon_drv(int drive) {
 #endif
 
 #ifdef HAVE_STRNCASECMP_L
-static int cmp_tok(const char *a, const char *b, int len) {
+static int cmp_tok(const char *a, const char *b, size_t len) {
     init_canon();
     return strncasecmp_l(a, b, len, C);
 }
 #else
-static int cmp_tok(const char *a, const char *b, int len) {
+static int cmp_tok(const char *a, const char *b, size_t len) {
     return strncasecmp(a, b, len);
 }
 #endif
@@ -274,6 +276,7 @@ static void get_env_conf(void)
 		* ((uint16_t *)global_switches[i].address) = strtou16(s,0,0);
 		break;
 	    case T_STRING:
+	    case T_UQSTRING:
 		* ((char **)global_switches[i].address) = s;
 		break;
 	    }
@@ -361,11 +364,21 @@ static char *get_string(void)
     end = strchr(str, '\"');
     if(!end)
 	syntax("unterminated string constant", 1);
-    *end = '\0';
+    str = strndup(str, ptrdiff(end, str));
     pos = end+1;
     return str;
 }
 
+static char *get_unquoted_string(void)
+{
+    if(*pos == '"')
+	return get_string();
+    else {
+	char *str=get_next_token();
+	return strndup(str, token_length);
+    }
+}
+
 static unsigned long get_unumber(unsigned long max)
 {
     char *last;
@@ -452,7 +465,7 @@ static void prepend(void)
 static void append(void)
 {
     grow();
-    cur_dev = cur_devs;
+    cur_dev = (int) cur_devs;
     cur_devs++;
     init_drive();
 }
@@ -512,7 +525,10 @@ static int set_var(struct switches_l *switches, int nr,
 		    get_number();
 	    else if (switches[i].type == T_STRING)
 		* ((char**)((long)switches[i].address+base_address))=
-		    strdup(get_string());
+		    get_string();
+	    else if (switches[i].type == T_UQSTRING)
+		* ((char**)((long)switches[i].address+base_address))=
+		    get_unquoted_string();
 	    return 0;
 	}
     }
@@ -594,7 +610,7 @@ void set_cmd_line_image(char *img) {
     devices[cur_dev].name = strdup(img);
     devices[cur_dev].offset = 0;
   } else {
-    devices[cur_dev].name = strndup(img, ofsp - img);
+    devices[cur_dev].name = strndup(img, ptrdiff(ofsp, img));
     devices[cur_dev].offset = str_to_offset(ofsp+2);
   }
 
@@ -639,7 +655,6 @@ static uint16_t tou16(int in, const char *comment) {
 	exit(1);
     }
     return (uint16_t) in;
-       
 }
 
 static void parse_old_device_line(char drive)
@@ -648,7 +663,7 @@ static void parse_old_device_line(char drive)
     int items;
     long offset;
 
-    int heads, sectors;
+    int heads, sectors, tracks;
     
     /* finish any old drive */
     finish_drive_clause();
@@ -660,11 +675,10 @@ static void parse_old_device_line(char drive)
     append();
     items = sscanf(token,"%c %s %i %i %i %i %li",
 		   &devices[cur_dev].drive,name,&devices[cur_dev].fat_bits,
-		   &devices[cur_dev].tracks,&heads,
-		   &sectors, &offset);
+		   &tracks,&heads,&sectors, &offset);
     devices[cur_dev].heads = tou16(heads, "heads");
     devices[cur_dev].sectors = tou16(sectors, "sectors");
-    
+    devices[cur_dev].tracks = (unsigned int) tracks;
     devices[cur_dev].offset = (off_t) offset;
     switch(items){
 	case 2:
