@@ -28,6 +28,7 @@
 #include "floppyd_io.h"
 #include "plain_io.h"
 
+static void usage(void) NORETURN;
 static void usage(void) 
 {
 	fprintf(stderr, "Mtools version %s, dated %s\n", 
@@ -38,21 +39,22 @@ static void usage(void)
 }
 
 #ifdef __CYGWIN__
-#define BUF_SIZE 512
+#define BUF_SIZE 512u
 #else
-#define BUF_SIZE 16000
+#define BUF_SIZE 16000u
 #endif
 
 static size_t bufLen(size_t blocksize, mt_size_t totalSize, mt_off_t address)
 {
 	if(totalSize == 0)
 		return blocksize;
-	if(address + blocksize > totalSize)
-		return totalSize - address;
+	if((mt_off_t) blocksize > (mt_off_t) totalSize - address)
+		return (size_t) ((mt_off_t)totalSize - address);
 	return blocksize;
 }
 
-void mcat(int argc, char **argv, int type)
+void mcat(int argc, char **argv, int type UNUSEDP) NORETURN;
+void mcat(int argc, char **argv, int type UNUSEDP)
 {
 	struct device *dev;
 	struct device out_dev;
@@ -65,7 +67,6 @@ void mcat(int argc, char **argv, int type)
 
 	char mode = O_RDONLY;
 	int optindex = 1;
-	size_t len;
 
 	noPrivileges = 1;
 
@@ -90,7 +91,7 @@ void mcat(int argc, char **argv, int type)
 		usage();
 	}
 
-        drive = toupper(argv[optindex][0]);
+        drive = ch_toupper(argv[optindex][0]);
 
         /* check out a drive whose letter and parameters match */       
         sprintf(errmsg, "Drive '%c:' not supported", drive);    
@@ -115,8 +116,8 @@ void mcat(int argc, char **argv, int type)
 
 #ifdef USE_FLOPPYD
                 if(!Stream)
-                        Stream = FloppydOpen(&out_dev, dev, name, 
-					     mode, errmsg, 0, 1);
+                        Stream = FloppydOpen(&out_dev, name, 
+					     mode, errmsg, NULL);
 #endif
 
 
@@ -138,22 +139,24 @@ void mcat(int argc, char **argv, int type)
 
 
 	if (mode == O_WRONLY) {
+		size_t len;
 		mt_size_t size=0;
 		size = out_dev.sectors * out_dev.heads * out_dev.tracks;
 		size *= 512;
 		while ((len = fread(buf, 1,
 				    bufLen(BUF_SIZE, size, address),
 				    stdin)) > 0) {			
-			int r = WRITES(Stream, buf, address, len);
+			ssize_t r = WRITES(Stream, buf, address, len);
 			fprintf(stderr, "Wrote to %d\n", (int) address);
 			if(r < 0)
 				break;
 			address += len;
 		}
 	} else {
+		ssize_t len;
 		while ((len = READS(Stream, buf, address, BUF_SIZE)) > 0) {
-			fwrite(buf, 1, len, stdout);
-			address += len;
+			fwrite(buf, 1, (size_t) len, stdout);
+			address += (size_t) len;
 		}
 	}
 
